unit IndexTabs;
{**********************************************************************}
{** IndexTabs - provides a index tab control in the style of Outlook **}
{**   		97 / Contacts Folder                      	     **}
{** ---------------------------------------------------------------- **}
{** Author  - Maurizio Panzica.                                      **}
{**   	      Portions of code were revised by Jordan Russell        **}
{** e-mail  - maurizio.panzica@orbit.net                             **}
{** ---------------------------------------------------------------- **}
{** Brief History                                                    **}
{**   - Version 1.00  (11/August/1997)  Initial release              **}
{** ---------------------------------------------------------------- **}
{** Copyright- copyright 1997 by Maurizio Panzica             	     **}
{**   Free to use and redistribute. No warranty is given by the      **}
{**   author, expressed or implied.                                  **}
{**********************************************************************}

interface

uses Messages, Windows, Classes, Controls, Graphics;

type
  { TIndexTabs definition -----------------------------------------------------}

  TButtonClickEvent = procedure (Sender: TObject; Index: integer;
                      ButtonCaption: string) of object;

  TIndexTabs = class(TCustomControl)
  private
    FUpperCase: boolean;
    FBtnHeight: integer;
    FBtnGap: integer;
    FButtons: array[0..26] of TRect;
    FClicked: boolean;
    FCurrBtn: integer;
    FTotBtn: integer;
    FOnButtonClick: TButtonClickEvent;
    FBorderWidth: integer;

    procedure DrawButton(Index: integer; Sunken: boolean);
    function GetBtnCaption(const BtnID, NumOfBtn: integer):string;
    procedure CalculateButtonsArray;
    procedure SetBtnGap(AValue:integer);
    procedure SetBorderWidth(AValue:integer);
    procedure SetUpperCase(AValue: boolean);

    procedure WMSize (var Message: TWMSize); message WM_SIZE;

  protected
    procedure CreateParams (var Params: TCreateParams); override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
                      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer);override;

  public
    constructor Create(AOwner: TComponent); override;

  published
    property ButtonGap:integer read FBtnGap write SetBtnGap;
    property BorderWidth:integer read FBorderWidth write SetBorderWidth;
    property OnButtonClick: TButtonClickEvent read FOnButtonClick 
             write FOnButtonClick;
    property UpperCase: boolean read FUpperCase write SetUpperCase;

    property Align;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;

    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;

  end;

procedure Register;

implementation
//===============================================================

procedure Register;
begin
  RegisterComponents('Freeware', [TIndexTabs]);
end;

constructor TIndexTabs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csCaptureMouse, csClickEvents, csOpaque, csReplicatable];
  Width:=35;
  Height:=100;
  FBtnGap:=2;
  FBorderWidth:=0;
end;

procedure TIndexTabs.CreateParams (var Params: TCreateParams);
begin
  inherited;
  with Params.WindowClass do
    Style := Style and not (CS_HREDRAW or CS_VREDRAW);
end;

procedure TIndexTabs.WMSize (var Message: TWMSize);
begin
  inherited;
  { Since the automatic erase/redraw is disabled when it is resized, it
    must manually repaint it }
  Paint;
end;

procedure TIndexTabs.Paint;
var
  i: integer;
begin
  inherited;
  with Canvas do
  begin
    FillRect(ClientRect);
    CalculateButtonsArray;
    for i:=0 to 26 do
      if i<FTotBtn then
        DrawButton(i,False)
      else
        FillRect(FButtons[i]);
  end;
end;

procedure TIndexTabs.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
Y: Integer);
var
  i: integer;
begin
  if Button=mbLeft then
    for i:=0 to FTotBtn -1 do
      if PtInRect(FButtons[i], Point(X,Y)) then
      begin
        FCurrBtn:=i;
        FClicked:=True;
        DrawButton(i,True);
      end;
  inherited;
end;

procedure TIndexTabs.MouseMove(Shift: TShiftState; X,Y: Integer);
begin
  if FClicked then
    DrawButton(FCurrBtn,PtInRect(FButtons[FCurrBtn],Point(X,Y)));
  inherited;
end;

procedure TIndexTabs.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y:
Integer);
begin
  if Button=mbLeft then
  begin
    if FCurrBtn<>-1 then
    begin
      DrawButton(FCurrBtn,False);
      if PtInRect(FButtons[FCurrBtn],Point(X,Y)) then
        if Assigned(FOnButtonClick) then
           FOnButtonClick( Self,
                           FCurrBtn,
                           GetBtnCaption(FCurrBtn, FTotBtn-1)
                          );
    end;
    FCurrBtn:=-1;
    FClicked:=False;
  end;
  inherited;
end;

function TIndexTabs.GetBtnCaption(const BtnID, NumOfBtn: integer):string;
var
  Res,
  BtnMask: string;
  ChrBase: integer;
  Dash: boolean;
  x: integer;
begin
  if FUpperCase then ChrBase:=64 else ChrBase:=96;

  if BtnID=0 then
    Result:='123'

  else
  begin
    BtnMask:='';
    case NumOfBtn of
      1:  Result:=chr(ChrBase+1)+'-'+chr(ChrBase+26);
      // Each number represents the button number
      // i.e.: if NumOfBtn=8, 4th button caption is JKL
      //            A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z
      2:  BtnMask:=#01#01#01#01#01#01#01#01#01#01#01#01#02#02#02#02#02#02#02#02#02#02#02#02#02#02;
      3:  BtnMask:=#01#01#01#01#01#01#01#01#01#02#02#02#02#02#02#02#02#02#03#03#03#03#03#03#03#03;
      4:  BtnMask:=#01#01#01#01#01#01#01#02#02#02#02#02#02#03#03#03#03#03#03#04#04#04#04#04#04#04;
      5:  BtnMask:=#01#01#01#01#01#01#02#02#02#02#02#02#03#03#03#03#03#04#04#04#05#05#05#05#05#05;
      6:  BtnMask:=#01#01#01#01#02#02#02#02#03#03#03#03#03#04#04#04#04#05#05#05#06#06#06#06#06#06;
      7:  BtnMask:=#01#01#01#01#02#02#02#02#03#03#03#03#04#04#04#05#05#05#05#06#06#06#07#07#07#07;
      8:  BtnMask:=#01#01#01#02#02#02#03#03#03#04#04#04#05#05#05#06#06#06#06#07#07#07#08#08#08#08;
      9:  BtnMask:=#01#01#01#02#02#02#03#03#04#04#05#05#06#06#06#07#07#07#07#08#08#08#09#09#09#09;
      10: BtnMask:=#01#01#01#02#02#02#03#03#04#04#05#05#06#06#06#07#07#08#08#09#09#09#10#10#10#10;
      11: BtnMask:=#01#01#02#02#03#03#04#04#05#05#06#06#07#07#07#08#08#09#09#10#10#10#11#11#11#11;
      12: BtnMask:=#01#01#02#02#03#03#04#04#05#05#06#06#07#07#07#08#08#09#09#10#10#10#11#11#12#12;
      13: BtnMask:=#01#01#02#02#03#03#04#04#05#05#06#06#07#07#07#08#08#09#09#10#11#11#12#12#13#13;
      14: BtnMask:=#01#01#02#02#03#03#04#04#05#05#06#06#07#08#08#09#09#10#11#11#12#12#13#13#14#14;
      15: BtnMask:=#01#01#02#03#04#04#05#05#06#06#07#07#08#09#09#10#10#11#12#12#13#13#14#14#15#15;
      16: BtnMask:=#01#01#02#03#04#04#05#05#06#06#07#07#08#09#09#10#10#11#12#13#14#14#15#15#16#16;
      17: BtnMask:=#01#01#02#03#04#04#05#05#06#06#07#07#08#09#10#11#11#12#13#14#15#15#16#16#17#17;
      18: BtnMask:=#01#02#03#04#05#05#06#06#07#07#08#08#09#10#11#12#12#13#14#15#16#16#17#17#18#18;
      19: BtnMask:=#01#02#03#04#05#05#06#06#07#07#08#09#10#11#12#13#13#14#15#16#17#17#18#18#19#19;
      20: BtnMask:=#01#02#03#04#05#05#06#07#08#08#09#10#11#12#13#14#14#15#16#17#18#18#19#19#20#20;
      21: BtnMask:=#01#02#03#04#05#06#07#08#09#09#10#11#12#13#14#15#15#16#17#18#19#19#20#20#21#21;
      22: BtnMask:=#01#02#03#04#05#06#07#08#09#10#11#12#13#14#15#16#16#17#18#19#20#20#21#21#22#22;
      23: BtnMask:=#01#02#03#04#05#06#07#08#09#10#11#12#13#14#15#16#16#17#18#19#20#21#22#22#23#23;
      24: BtnMask:=#01#02#03#04#05#06#07#08#09#10#11#12#13#14#15#16#16#17#18#19#20#21#22#22#23#24;
      25: BtnMask:=#01#02#03#04#05#06#07#08#09#10#11#12#13#14#15#16#16#17#18#19#20#21#22#23#24#25;
      26: Result:=chr(ChrBase+BtnID);
    end;
    if BtnMask='' then exit;
    //Use dash to separate the first and the last character, if less then 8 btns
    Dash:=(NumOfBtn<8);
    x:=pos(chr(BtnID),BtnMask);
    //Add the first character
    if x>0 then res:=chr(x+ChrBase);
    while BtnMask[x+1]=chr(BtnID) do
    begin
      Inc(x);
      if not Dash then res:=res+chr(x+ChrBase);
    end;
    if Dash then res:=res+'-'+chr(x+ChrBase);
    Result:=res;
  end;
end;

procedure TIndexTabs.DrawButton(Index: integer; Sunken: boolean);
var
  Offset,
  Flags: integer;
  TempRect: TRect;
  s: string;
  w: integer;
begin
  if Sunken then
  begin
    Flags:=BDR_SUNKENOUTER;
    Offset:=1;
  end

  else
  begin
    Flags:=BDR_RAISEDINNER;
    Offset:=0;
  end;
  TempRect:=FButtons[Index];
  DrawEdge(Canvas.Handle,TempRect,Flags, BF_RECT);
  s:=GetBtnCaption(Index,FTotBtn-1);
  InflateRect(TempRect,-1,-1);
  w:=(TempRect.Right-TempRect.Left-Canvas.TextWidth(s)) div 2;
  Canvas.Brush.Color:=Self.Color;
  Canvas.Font:=Self.Font;
  Canvas.TextRect(TempRect,
                  TempRect.Left+w+Offset,
                  TempRect.Top+Offset,
                  s);
end;

procedure TIndexTabs.CalculateButtonsArray;
var
  t,
  i: integer;
begin
  FBtnHeight:=Canvas.TextHeight('w')+4;
  for i:=0 to 26 do
  begin
    t:=(i*(FBtnHeight+FBtnGap))+FBorderWidth;
    FButtons[i]:=Rect(FBorderWidth,
                      t,
                      ClientWidth-(FBorderWidth),
                      t+FBtnHeight
                     );
  end;
  FTotBtn:=(ClientHeight+FBtnGap-(FBorderWidth*2)) div (FBtnHeight+FBtnGap);
  if FTotBtn > 27 then FTotBtn:=27;
  if FTotBtn < 2 then FTotBtn:=2;
end;

procedure TIndexTabs.SetBtnGap(AValue:integer);
begin
  if AValue <> FBtnGap then
  begin
    if AValue<0 then AValue:=0;
    if AValue>16 then AValue:=16;
    FBtnGap:=AValue;
    Paint;
  end;
end;

procedure TIndexTabs.SetBorderWidth(AValue:integer);
begin
  if AValue <> FBorderWidth then
  begin
    if AValue<0 then AValue:=0;
    if AValue>16 then AValue:=16;
    FBorderWidth:=AValue;
    Paint;
  end;
end;

procedure TIndexTabs.SetUpperCase(AValue: boolean);
begin
  if AValue<> FUpperCase then
  begin
    FUpperCase:=AValue;
    Paint;
  end;
end;

end.
